/* support routines for code generated by the util programs */

#include <stdio.h>
#include <string.h>

void dump_int (int i)
{
  fprintf (stdout, ">>%d<<\n", i);
}

void dump_long (long l)
{
  fprintf (stdout, ">>%ld<<\n", l);
}

void dump_string (char *s)
{
  if ((s == NULL) || (strlen (s) == 0))
    fprintf (stdout, "(null)\n");
  else
    fprintf (stdout, ">>%s<<\n", s);
}
